/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch das
 * Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.formcycle;

import static de.ozgcloud.formcycle.HttpPostRequestBuilder.ATTACHMENT_KEY;
import static de.ozgcloud.formcycle.HttpPostRequestBuilder.FORM_DATA_KEY;
import static de.ozgcloud.formcycle.HttpPostRequestBuilder.PROTOBUF_CONTENT_TYPE;
import static de.ozgcloud.formcycle.HttpPostRequestBuilder.REPRESENTATION_KEY;
import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.net.URI;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.itvsh.kop.eingangsadapter.formcycle.FormCycleFormData;
import de.ozgcloud.formcycle.attachment.FormcycleAttachmentTestFactory;
import lombok.SneakyThrows;

class HttpPostRequestBuilderTest {

	static final String TARGET_URL = "http://url";

	@Spy
	@InjectMocks
	private HttpPostRequestBuilder requestBuilder;

	@Mock
	private OzgCloudConfig cloudConfig;
	@Mock
	private MultipartEntityBuilder entityBuilder;

	@SneakyThrows
	@BeforeEach
	void setup() {
		var entityBuilderField = requestBuilder.getClass().getDeclaredField("entityBuilder");
		entityBuilderField.setAccessible(true);
		entityBuilderField.set(requestBuilder, entityBuilder);
	}

	@Nested
	class TestAddFormData {

		@Test
		@DisplayName("should add form data http request")
		void shouldAddFormData() {
			var formDataContent = new byte[] {};
			FormCycleFormData formData = when(mock(FormCycleFormData.class).toByteArray()).thenReturn(formDataContent).getMock();

			requestBuilder.withFormData(formData);

			verify(entityBuilder).addBinaryBody(FORM_DATA_KEY, formDataContent, PROTOBUF_CONTENT_TYPE, null);
		}

		@Test
		@DisplayName("should return same object")
		void shouldReturnSameObject() {
			var formData = mock(FormCycleFormData.class);

			var httpPostRequestBuilder = requestBuilder.withFormData(formData);

			assertThat(httpPostRequestBuilder).isEqualTo(requestBuilder);
		}
	}

	@Nested
	@DisplayName("Builder with attachments")
	class TestAddAttachments {

		@Test
		@DisplayName("should add attachments http request")
		void shouldAddFormData() {
			var attachment = FormcycleAttachmentTestFactory.create();

			requestBuilder.withAttachments(List.of(attachment));

			verify(entityBuilder).addBinaryBody(ATTACHMENT_KEY, attachment.getContent(), attachment.getContentType(), attachment.getFileName());
		}

		@Test
		@DisplayName("should return same object")
		void shouldReturnSameObject() {
			var attachment = FormcycleAttachmentTestFactory.create();

			var httpPostRequestBuilder = requestBuilder.withAttachments(List.of(attachment));

			assertThat(httpPostRequestBuilder).isEqualTo(requestBuilder);
		}

	}

	@Nested
	@DisplayName("Builder with representations")
	class TestAddRepresentations {

		@Test
		@DisplayName("should add representations to http request")
		void shouldAddFormData() {
			var representation = FormcycleAttachmentTestFactory.create();

			requestBuilder.withRepresentations(List.of(representation));

			verify(entityBuilder).addBinaryBody(REPRESENTATION_KEY, representation.getContent(), representation.getContentType(),
					representation.getFileName());
		}

		@Test
		@DisplayName("should return same object")
		void shouldReturnSameObject() {
			var representation = FormcycleAttachmentTestFactory.create();

			var httpPostRequestBuilder = requestBuilder.withRepresentations(List.of(representation));

			assertThat(httpPostRequestBuilder).isEqualTo(requestBuilder);
		}

	}

	@Nested
	class TestBuildPostRequest {


		@BeforeEach
		void setup() {
			when(cloudConfig.getEingangsAdapterUrl()).thenReturn(TARGET_URL);
		}

		@Test
		void shouldSetTargetUrl() {
			var	expectedUrl = TARGET_URL + "/" + HttpPostRequestBuilder.FORMCYCLE_ENDPOINT;
			when(cloudConfig.getEingangsAdapterUrl()).thenReturn(TARGET_URL);

			var httpPost = requestBuilder.build();

			assertThat(httpPost.getURI()).isEqualTo(URI.create(expectedUrl));
		}

		@Test
		void shouldSetEntity() {
			var expectedHttpEntity = mock(HttpEntity.class);
			when(entityBuilder.build()).thenReturn(expectedHttpEntity);

			var httpPost = requestBuilder.build();

			assertThat(httpPost.getEntity()).isEqualTo(expectedHttpEntity);
		}
	}

	@Nested
	class TestBuildEingangsAdapterUrl {

		@Test
		void shouldAddEndpointWithSlash() {
			when(cloudConfig.getEingangsAdapterUrl()).thenReturn(TARGET_URL);

			var url = requestBuilder.buildEingagsAdapterUrl();

			assertThat(url).isEqualTo(TARGET_URL + "/" + HttpPostRequestBuilder.FORMCYCLE_ENDPOINT);
		}

		@Test
		void shouldAddEndpoint() {
			var expectedUrl = TARGET_URL + "url/";
			when(cloudConfig.getEingangsAdapterUrl()).thenReturn(expectedUrl);

			var url = requestBuilder.buildEingagsAdapterUrl();

			assertThat(url).isEqualTo(expectedUrl + HttpPostRequestBuilder.FORMCYCLE_ENDPOINT);
		}

		@Test
		void shouldAcceptUrlWithEndpoint() {
			var expectedUrl = TARGET_URL + "/" + HttpPostRequestBuilder.FORMCYCLE_ENDPOINT;
			when(cloudConfig.getEingangsAdapterUrl()).thenReturn(expectedUrl);

			var url = requestBuilder.buildEingagsAdapterUrl();

			assertThat(url).isEqualTo(expectedUrl);
		}
	}
}